from collections.abc import Callable, Mapping, Sequence
from typing import Annotated, overload

from numpy.typing import ArrayLike


class AudioPackets:
    def __repr__(self) -> str: ...

    @property
    def timestamp(self) -> tuple[float, float] | None: ...

    @property
    def sample_rate(self) -> int: ...

    @property
    def num_channels(self) -> int: ...

    def clone(self) -> AudioPackets: ...

class CPUBuffer:
    @property
    def __array_interface__(self) -> dict: ...

class CPUStorage:
    pass

class CUDABuffer:
    @property
    def __cuda_array_interface__(self) -> None: ...

    @property
    def device_index(self) -> None: ...

class CUDAConfig:
    def __init__(self, device_index: int, stream: int = 0, allocator: tuple[Callable[[int, int, int], int], Callable[[int], None]] | None = None) -> None: ...

class DecodeConfig:
    def __init__(self, decoder: str | None = None, decoder_options: Mapping[str, str] | None = None) -> None: ...

class DemuxConfig:
    def __init__(self, format: str | None = None, format_options: Mapping[str, str] | None = None, buffer_size: int = 8096) -> None: ...

class Demuxer:
    def demux_audio(self, window: tuple[float, float] | None = None, bsf: str | None = None) -> AudioPackets: ...

    def demux_video(self, window: tuple[float, float] | None = None, bsf: str | None = None) -> VideoPackets: ...

    def demux_image(self, bsf: str | None = None) -> ImagePackets: ...

    def has_audio(self) -> bool: ...

class EncodeConfig:
    def __init__(self, muxer: str | None = None, muxer_options: Mapping[str, str] | None = None, encoder: str | None = None, encoder_options: Mapping[str, str] | None = None, format: str | None = None, width: int = -1, height: int = -1, scale_algo: str | None = None, filter_desc: str | None = None, bit_rate: int = -1, compression_level: int = -1, qscale: int = -1, gop_size: int = -1, max_bframes: int = -1) -> None: ...

class FFmpegAudioFrames:
    @property
    def num_frames(self) -> int: ...

    @property
    def sample_rate(self) -> int: ...

    @property
    def num_channels(self) -> int: ...

    @property
    def sample_fmt(self) -> str: ...

    def __len__(self) -> int: ...

    def __repr__(self) -> str: ...

    def clone(self) -> FFmpegAudioFrames: ...

class FFmpegImageFrames:
    @property
    def num_planes(self) -> int: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    @property
    def pix_fmt(self) -> str: ...

    @property
    def metadata(self) -> dict[str, str]: ...

    def __repr__(self) -> str: ...

    def clone(self) -> FFmpegImageFrames: ...

    @property
    def pts(self) -> float: ...

class FFmpegVideoFrames:
    @property
    def num_frames(self) -> int: ...

    @property
    def num_planes(self) -> int: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    @property
    def pix_fmt(self) -> str: ...

    def __len__(self) -> int: ...

    @overload
    def __getitem__(self, arg: slice, /) -> FFmpegVideoFrames: ...

    @overload
    def __getitem__(self, arg: int, /) -> FFmpegImageFrames: ...

    @overload
    def __getitem__(self, arg: Sequence[int], /) -> FFmpegVideoFrames: ...

    def __repr__(self) -> str: ...

    def clone(self) -> FFmpegVideoFrames: ...

class ImagePackets:
    @property
    def pix_fmt(self) -> str: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    def __repr__(self) -> str: ...

    def clone(self) -> ImagePackets: ...

class InternalError(AssertionError):
    pass

class StreamingVideoDecoder:
    def decode(self, arg: int, /) -> FFmpegVideoFrames | None: ...

class TracingSession:
    def init(self) -> None: ...

    def config(self, arg: str, /) -> None: ...

    def start(self, arg0: int, arg1: int, /) -> None: ...

    def stop(self) -> None: ...

class VideoPackets:
    @property
    def timestamp(self) -> tuple[float, float] | None: ...

    @property
    def pix_fmt(self) -> str: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    @property
    def frame_rate(self) -> tuple[int, int]: ...

    def __len__(self) -> int: ...

    def __repr__(self) -> str: ...

    def clone(self) -> VideoPackets: ...

def convert_array(vals: Annotated[ArrayLike, dict(dtype='int64', order='C', device='cpu')], storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: FFmpegAudioFrames, storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: FFmpegVideoFrames, storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: FFmpegImageFrames, storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: Sequence[FFmpegAudioFrames], storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: Sequence[FFmpegVideoFrames], storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: Sequence[FFmpegImageFrames], storage: CPUStorage | None = None) -> CPUBuffer: ...

def cpu_storage(size: int, pin_memory: bool = False) -> CPUStorage: ...

@overload
def decode_image_nvjpeg(data: bytes, *, device_config: CUDAConfig, scale_width: int = -1, scale_height: int = -1, pix_fmt: str = 'rgb', _zero_clear: bool = False) -> CUDABuffer: ...

@overload
def decode_image_nvjpeg(data: Sequence[bytes], *, device_config: CUDAConfig, scale_width: int, scale_height: int, pix_fmt: str = 'rgb', _zero_clear: bool = False) -> CUDABuffer: ...

@overload
def decode_packets(packets: AudioPackets, *, decode_config: DecodeConfig | None = None, filter_desc: str | None = None) -> FFmpegAudioFrames: ...

@overload
def decode_packets(packets: VideoPackets, *, decode_config: DecodeConfig | None = None, filter_desc: str | None = None) -> FFmpegVideoFrames: ...

@overload
def decode_packets(packets: ImagePackets, *, decode_config: DecodeConfig | None = None, filter_desc: str | None = None) -> FFmpegImageFrames: ...

@overload
def decode_packets_nvdec(packets: VideoPackets, *, device_config: CUDAConfig, crop_left: int = 0, crop_top: int = 0, crop_right: int = 0, crop_bottom: int = 0, width: int = -1, height: int = -1, pix_fmt: str | None = 'rgba') -> CUDABuffer: ...

@overload
def decode_packets_nvdec(packets: ImagePackets, *, device_config: CUDAConfig, crop_left: int = 0, crop_top: int = 0, crop_right: int = 0, crop_bottom: int = 0, width: int = -1, height: int = -1, pix_fmt: str | None = 'rgba') -> CUDABuffer: ...

@overload
def decode_packets_nvdec(packets: Sequence[ImagePackets], *, device_config: CUDAConfig, crop_left: int = 0, crop_top: int = 0, crop_right: int = 0, crop_bottom: int = 0, width: int = -1, height: int = -1, pix_fmt: str | None = 'rgba', strict: bool = True) -> CUDABuffer: ...

@overload
def encode_image(path: str, data: Annotated[ArrayLike, dict(order='C', device='cpu')], *, pix_fmt: str = 'rgb24', encode_config: EncodeConfig | None = None) -> None: ...

@overload
def encode_image(path: str, data: Annotated[ArrayLike, dict(dtype='uint8', order='C', device='cuda')], *, pix_fmt: str = 'rgb24', encode_config: EncodeConfig | None = None) -> None: ...

def get_ffmpeg_filters() -> list[str]: ...

def get_ffmpeg_log_level() -> int: ...

def init_glog(arg: str, /) -> None: ...

def init_tracing() -> TracingSession: ...

def is_cuda_available() -> bool: ...

def is_nvcodec_available() -> bool: ...

def is_nvjpeg_available() -> bool: ...

def register_avdevices() -> None: ...

def set_ffmpeg_log_level(arg: int, /) -> None: ...

@overload
def trace_counter(arg0: int, arg1: int, /) -> None: ...

@overload
def trace_counter(arg0: int, arg1: float, /) -> None: ...

def trace_event_begin(arg: str, /) -> None: ...

def trace_event_end() -> None: ...

@overload
def transfer_buffer(buffer: CPUBuffer, *, device_config: CUDAConfig) -> CUDABuffer: ...

@overload
def transfer_buffer(buffer: Annotated[ArrayLike, dict(order='C', device='cpu')], *, device_config: CUDAConfig) -> CUDABuffer: ...

def transfer_buffer_cpu(buffer: Annotated[ArrayLike, dict(order='C', device='cuda')]) -> CPUBuffer: ...
